@@ -15,10 +15,9 @@
 
 #include "config.hh"
 
-#include <cerrno>
+#include <algorithm>
 #include <climits>
-#include <cstdlib>
-#include <limits>
+#include <cstddef>
 #include <sstream>
 #include <stdexcept>
 
@@ -28,157 +27,11 @@
 #include "debug.hh"
 #include "djvu-const.hh"
 #include "i18n.hh"
+#include "string-printf.hh"
+#include "string-utils.hh"
 #include "system.hh"
 
-class OptionsParseError : public Config::Error
-{
-protected:
-  explicit OptionsParseError(const std::string &message)
-  : Config::Error(message)
-  { }
-public:
-  OptionsParseError()
-  : Config::Error(_("Unable to parse command-line options"))
-  { }
-};
-
-class NoInputFile : public Config::Error
-{
-public:
-  NoInputFile()
-  : Config::Error(_("No input file name was specified"))
-  { }
-};
-
-class InvalidOutputFileName : public Config::Error
-{
-public:
-  InvalidOutputFileName()
-  : Config::Error(_("Invalid output file name"))
-  { }
-};
-
-class PagesParseError : public Config::Error
-{
-public:
-  PagesParseError()
-  : Config::Error(_("Unable to parse page numbers"))
-  { }
-};
-
-class PageSizeParseError : public Config::Error
-{
-public:
-  PageSizeParseError()
-  : Config::Error(_("Unable to parse page size"))
-  { }
-};
-
-class HyperlinksOptionsParseError : public Config::Error
-{
-public:
-  HyperlinksOptionsParseError()
-  : Config::Error(_("Unable to parse hyperlinks options"))
-  { }
-};
-
-class DpiOutsideRange : public Config::Error
-{
-public:
-  DpiOutsideRange(int dpi_from, int dpi_to)
-  : Config::Error(string_printf(_("The specified resolution is outside the allowed range: %d .. %d"), dpi_from, dpi_to))
-  { }
-};
-
-class FgColorsOutsideRange : public Config::Error
-{
-public:
-  FgColorsOutsideRange(unsigned int n, unsigned int m)
-  : Config::Error(string_printf(_("The specified number of foreground colors is outside the allowed range: %u .. %u"), n, m))
-  { }
-};
-
-class SubsampleRatioOutsideRange : public Config::Error
-{
-public:
-  SubsampleRatioOutsideRange(unsigned int n, unsigned int m)
-  : Config::Error(string_printf(_("The specified subsampling ratio is outside the allowed range: %u .. %u"), n, m))
-  { }
-};
-
-class LossLevelWithoutMonochrome : public Config::Error
-{
-public:
-  LossLevelWithoutMonochrome()
-  : Config::Error(_("--loss-level requires enabling --monochrome"))
-  { }
-};
-
-class InputOutputClash : public Config::Error
-{
-public:
-  explicit InputOutputClash(const std::string &path)
-  : Config::Error(string_printf(_("Input file is the same as output file: %s"), path.c_str()))
-  { }
-};
-
-class PageTitleEncodingError : public Config::Error
-{
-public:
-  explicit PageTitleEncodingError(const std::runtime_error &exc)
-  : Config::Error(string_printf(_("Unable to convert page title to UTF-8: %s"), exc.what()))
-  { }
-};
-
-class PageTitleTemplateParseError : public Config::Error
-{
-public:
-  PageTitleTemplateParseError()
-  : Config::Error(_("Unable to parse page title template specification"))
-  { }
-};
-
-class PageidTemplateParseError : public Config::Error
-{
-public:
-  PageidTemplateParseError()
-  : Config::Error(_("Unable to parse pageid template specification"))
-  { }
-};
-
-class PageidIllegalCharacter : public Config::Error
-{
-public:
-  PageidIllegalCharacter()
-  : Config::Error(_("Pageid must consist only of letters, digits, '_', '+', '-' and '.' characters"))
-  { }
-};
-
-class PageidIllegalDot : public Config::Error
-{
-public:
-  PageidIllegalDot()
-  : Config::Error(_("Pageid cannot start with a '.' character or contain two consecutive '.' characters"))
-  { }
-};
-
-class PageidIllegalPm : public Config::Error
-{
-public:
-  PageidIllegalPm()
-  : Config::Error(_("Pageid cannot start with a '+' or a '-' character"))
-  { }
-};
-
-class PageidBadExtension : public Config::Error
-{
-public:
-  PageidBadExtension()
-  : Config::Error(_("Pageid must end with the '.djvu' or the '.djv' extension"))
-  { }
-};
-
-string_format::Template* Config::default_pageid_template(const std::string &prefix)
+string_format::Template* Config::default_page_id_template(const std::string &prefix)
 {
   return new string_format::Template(prefix + "{spage:04*}.djvu");
 }
@@ -205,57 +58,22 @@ Config::Config()
   this->monochrome = false;
   this->loss_level = 0;
   this->bg_slices = NULL;
-  this->pageid_template.reset(default_pageid_template("p"));
+  this->page_id_template.reset(default_page_id_template("p"));
+  this->page_title_template.reset(new string_format::Template("{label}"));
   this->n_jobs = 1;
 }
 
 namespace string
 {
-  static void split(const std::string &, char, std::vector<std::string> &);
-
-  static void replace(std::string &, char, char);
-
-  template <typename tp>
-  tp as(const char *);
-
-  template <>
-  long as<long>(const char *);
-
   template <typename tp>
   tp as(const std::string &);
 }
 
-static void string::split(const std::string &s, char c, std::vector<std::string> &result)
-{
-  size_t lpos = 0;
-  while (true)
-  {
-    size_t rpos = s.find(c, lpos);
-    result.push_back(s.substr(lpos, rpos - lpos));
-    if (rpos == std::string::npos)
-      break;
-    else
-      lpos = rpos + 1;
-  }
-}
-
-static void string::replace(std::string &s, char c1, char c2)
-{
-  size_t lpos = 0;
-  while (true)
-  {
-    size_t rpos = s.find(c1, lpos);
-    if (rpos == std::string::npos)
-      break;
-    s[rpos] = c2;
-    lpos = rpos + 1;
-  }
-}
 
 static void parse_hyperlinks_options(std::string s, Config::Hyperlinks &options)
 {
   std::vector<std::string> split;
-  string::replace(s, '_', '-');
+  std::replace(s.begin(), s.end(), '_', '-');
   string::split(s, ',', split);
   for (std::vector<std::string>::const_iterator it = split.begin(); it != split.end(); it++)
   {
@@ -279,10 +97,15 @@ static void parse_hyperlinks_options(std::string s, Config::Hyperlinks &options)
       options.border_color = *it;
       continue;
     }
-    throw HyperlinksOptionsParseError();
+    throw Config::Error(_("Unable to parse hyperlinks options"));
   }
 }
 
+static void bad_pages()
+{
+  throw Config::Error(_("Unable to parse page numbers"));
+}
+
 static void parse_pages(const std::string &s, std::vector< std::pair<int, int> > &result)
 {
   int state = 0;
@@ -303,18 +126,18 @@ static void parse_pages(const std::string &s, std::vector< std::pair<int, int> >
     else if (*it == ',')
     {
       if (value[0] < 1 || value[1] < 1 || value[0] > value[1])
-        throw PagesParseError();
+        bad_pages();
       result.push_back(std::make_pair(value[0], value[1]));
       value[0] = value[1] = 0;
       state = 0;
     }
     else
-      throw PagesParseError();
+      bad_pages();
   }
   if (state == 0)
     value[1] = value[0];
   if (value[0] < 1 || value[1] < 1 || value[0] > value[1])
-    throw PagesParseError();
+    bad_pages();
   result.push_back(std::make_pair(value[0], value[1]));
 }
 
@@ -327,44 +150,21 @@ static std::pair<int, int>parse_page_size(const std::string &s)
   if (x > 0 &&  y > 0 && c == 'x' && stream.eof() && !stream.fail())
     return std::make_pair(x, y);
   else
-    throw PageSizeParseError();
-}
-
-class InvalidNumber : public Config::Error
-{
-public:
-  explicit InvalidNumber(const char *s)
-  : Config::Error(string_printf(_("%s is not a valid number"), s))
-  { }
-};
-
-template <typename tp>
-tp string::as(const char *s)
-{
-  long result = as<long>(s);
-  if (result > std::numeric_limits<tp>::max())
-    throw InvalidNumber(s);
-  if (result < std::numeric_limits<tp>::min())
-    throw InvalidNumber(s);
-  return result;
-}
-
-template <>
-long string::as<long>(const char *s)
-{
-  char *end_ptr;
-  long result;
-  errno = 0;
-  result = strtol(s, &end_ptr, 10);
-  if (*end_ptr != '\0' || errno == ERANGE)
-    throw InvalidNumber(s);
-  return result;
+    throw Config::Error(_("Unable to parse page size"));
 }
 
 template <typename tp>
 tp string::as(const std::string &s)
 {
-  return as<tp>(s.c_str());
+  tp n;
+  std::istringstream stream(s);
+  stream >> n;
+  if (stream.fail() || !stream.eof())
+    throw Config::Error(string_printf(
+      _("\"%s\" is not a valid number"),
+      s.c_str())
+    );
+  return n;
 }
 
 static unsigned int parse_fg_colors(const std::string &s)
@@ -377,7 +177,10 @@ static unsigned int parse_fg_colors(const std::string &s)
     return Config::FG_COLORS_BLACK;
   long n = string::as<long>(s);
   if (n < 1 || n > static_cast<long>(djvu::max_fg_colors))
-    throw FgColorsOutsideRange(1, djvu::max_fg_colors);
+    throw Config::Error(string_printf(
+      _("The specified number of foreground colors is outside the allowed range: %u .. %u"),
+      1U, djvu::max_fg_colors
+    ));
   return n;
 }
 
@@ -385,41 +188,50 @@ static unsigned int parse_bg_subsample(const std::string &s)
 {
   long n = string::as<long>(optarg);
   if (n < 1 || n > static_cast<long>(djvu::max_subsample_ratio))
-    throw SubsampleRatioOutsideRange(1, djvu::max_subsample_ratio);
+    throw Config::Error(string_printf(
+      _("The specified subsampling ratio is outside the allowed range: %u .. %u"),
+      1, djvu::max_subsample_ratio
+    ));
   return n;
 }
 
-static void validate_pageid_template(string_format::Template &pageid_template)
+static void validate_page_id_template(string_format::Template &page_id_template)
 {
-  string_format::Bindings empty_bindings;
-  std::string pageid = pageid_template.format(empty_bindings);
-  size_t length = pageid.length();
+  string_format::Bindings bindings;
+  bindings["max_spage"] = 0U;
+  bindings["spage"] = 0U;
+  bindings["max_page"] = 0U;
+  bindings["page"] = 0U;
+  bindings["max_dpage"] = 0U;
+  bindings["dpage"] = 0U;
+  std::string page_id = page_id_template.format(bindings);
+  size_t length = page_id.length();
   bool dot_allowed = false;
   bool pm_allowed = false;
-  for (std::string::const_iterator it = pageid.begin(); it != pageid.end(); it++)
+  for (std::string::const_iterator it = page_id.begin(); it != page_id.end(); it++)
   {
     if (!pm_allowed && (*it == '+' || *it == '-'))
     {
-      throw PageidIllegalPm();
+      throw Config::Error(_("Page identifier cannot start with a '+' or a '-' character"));
     }
     if (*it == '.')
     {
       if (!dot_allowed)
-        throw PageidIllegalDot();
+        throw Config::Error(_("Page identifier cannot start with a '.' character or contain two consecutive '.' characters"));
       dot_allowed = false;
     }
     else if ((*it >= 'a' && *it <= 'z') || (*it >= '0' && *it <= '9') || *it == '_' || *it == '-' || *it == '+')
       dot_allowed = true;
     else
-      throw PageidIllegalCharacter();
+      throw Config::Error(_("Page identifier must consist only of letters, digits, '_', '+', '-' and '.' characters"));
     pm_allowed = true;
   }
-  if (length >= 4 && pageid.substr(length - 4) == ".djv")
+  if (length >= 4 && page_id.substr(length - 4) == ".djv")
     ;
-  else if (length >= 5 && pageid.substr(length - 5) == ".djvu")
+  else if (length >= 5 && page_id.substr(length - 5) == ".djvu")
     ;
   else
-    throw PageidBadExtension();
+    throw Config::Error(_("Page identifier must end with the '.djvu' or the '.djv' extension"));
 }
 
 void Config::read_config(int argc, char * const argv[])
@@ -448,9 +260,10 @@ void Config::read_config(int argc, char * const argv[])
     OPT_NO_HLINKS,
     OPT_NO_METADATA,
     OPT_NO_OUTLINE,
+    OPT_NO_PAGE_TITLES,
     OPT_NO_RENDER,
-    OPT_PAGEID_PREFIX,
-    OPT_PAGEID_TEMPLATE,
+    OPT_PAGE_ID_PREFIX,
+    OPT_PAGE_ID_TEMPLATE,
     OPT_PAGE_SIZE,
     OPT_PAGE_TITLE_TEMPLATE,
     OPT_TEXT_CROP,
@@ -487,13 +300,16 @@ void Config::read_config(int argc, char * const argv[])
     { "no-metadata", 0, 0, OPT_NO_METADATA },
     { "no-nfkc", 0, 0, OPT_TEXT_NO_NFKC },
     { "no-outline", 0, 0, OPT_NO_OUTLINE },
+    { "no-page-titles", 0, 0, OPT_NO_PAGE_TITLES },
     { "no-render", 0, 0, OPT_NO_RENDER },
     { "no-text", 0, 0, OPT_TEXT_NONE },
     { "output", 1, 0, OPT_OUTPUT },
+    { "page-id-prefix", 1, 0, OPT_PAGE_ID_PREFIX },
+    { "page-id-template", 1, 0, OPT_PAGE_ID_TEMPLATE },
     { "page-size", 1, 0, OPT_PAGE_SIZE },
     { "page-title-template", 1, 0, OPT_PAGE_TITLE_TEMPLATE },
-    { "pageid-prefix", 1, 0, OPT_PAGEID_PREFIX },
-    { "pageid-template", 1, 0, OPT_PAGEID_TEMPLATE },
+    { "pageid-prefix", 1, 0, OPT_PAGE_ID_PREFIX }, /* deprecated alias */
+    { "pageid-template", 1, 0, OPT_PAGE_ID_TEMPLATE }, /* deprecated alias */
     { "pages", 1, 0, OPT_PAGES },
     { "quiet", 0, 0, OPT_QUIET },
     { "verbatim-metadata", 0, 0, OPT_VERBATIM_METADATA },
@@ -510,13 +326,16 @@ void Config::read_config(int argc, char * const argv[])
     if (c < 0)
       break;
     if (c == 0)
-      throw OptionsParseError();
+      throw Config::Error(_("Unable to parse command-line options"));
     switch (c)
     {
     case OPT_DPI:
       this->dpi = string::as<int>(optarg);
       if (this->dpi < djvu::min_dpi || this->dpi > djvu::max_dpi)
-        throw DpiOutsideRange(djvu::min_dpi, djvu::max_dpi);
+        throw Config::Error(string_printf(
+          _("The specified resolution is outside the allowed range: %d .. %d"),
+          djvu::min_dpi, djvu::max_dpi
+        ));
       break;
     case OPT_GUESS_DPI:
       this->guess_dpi = true;
@@ -617,7 +436,7 @@ void Config::read_config(int argc, char * const argv[])
           /* ``djvmcvt`` does not support ``-`` as a file name.
            * Work-arounds are possible but not worthwhile.
            */
-          throw InvalidOutputFileName();
+          throw Config::Error(_("Invalid output file name"));
         }
       }
       break;
@@ -626,20 +445,20 @@ void Config::read_config(int argc, char * const argv[])
       this->output = optarg;
       this->output_stdout = false;
       break;
-    case OPT_PAGEID_PREFIX:
-      this->pageid_template.reset(this->default_pageid_template(optarg));
-      validate_pageid_template(*this->pageid_template);
+    case OPT_PAGE_ID_PREFIX:
+      this->page_id_template.reset(this->default_page_id_template(optarg));
+      validate_page_id_template(*this->page_id_template);
       break;
-    case OPT_PAGEID_TEMPLATE:
+    case OPT_PAGE_ID_TEMPLATE:
       try
       {
-        this->pageid_template.reset(new string_format::Template(optarg));
+        this->page_id_template.reset(new string_format::Template(optarg));
       }
       catch (string_format::ParseError)
       {
-        throw PageidTemplateParseError();
+        throw Config::Error(_("Unable to parse page identifier template specification"));
       }
-      validate_pageid_template(*this->pageid_template);
+      validate_page_id_template(*this->page_id_template);
       break;
     case OPT_PAGE_TITLE_TEMPLATE:
       try
@@ -652,13 +471,21 @@ void Config::read_config(int argc, char * const argv[])
       }
       catch (encoding::Error &exc)
       {
-        throw PageTitleEncodingError(exc);
+        throw Config::Error(string_printf(
+          _("Unable to convert page title to UTF-8: %s"),
+          exc.what())
+        );
       }
       catch (string_format::ParseError)
       {
-        throw PageTitleTemplateParseError();
+        throw Config::Error(
+          _("Unable to parse page title template specification")
+        );
       }
       break;
+    case OPT_NO_PAGE_TITLES:
+      this->page_title_template.reset(new string_format::Template(""));
+      break;
     case OPT_JOBS:
       this->n_jobs = string::as<int>(optarg);
       break;
@@ -674,15 +501,18 @@ void Config::read_config(int argc, char * const argv[])
     }
   }
   if (this->loss_level > 0 && !this->monochrome)
-    throw LossLevelWithoutMonochrome();
+    throw Config::Error(_("--loss-level requires enabling --monochrome"));
   if (optind > argc - 1)
-    throw NoInputFile();
+    throw Config::Error(_("No input file name was specified"));
   else
     while (optind < argc)
     {
       this->filenames.push_back(argv[optind]);
       if (is_same_file(this->output, argv[optind]))
-        throw InputOutputClash(this->output);
+        throw Config::Error(string_printf(
+          _("Input file is the same as output file: %s"),
+          this->output.c_str()
+        ));
       optind++;
     }
 }
@@ -701,9 +531,10 @@ void Config::usage(const Config::Error &error) const
     << std::endl << _("Options: ")
     << std::endl << _(" -i, --indirect=FILE")
     << std::endl << _(" -o, --output=FILE")
-    << std::endl << _("     --pageid-prefix=NAME")
-    << std::endl << _("     --pageid-template=TEMPLATE")
+    << std::endl << _("     --page-id-prefix=NAME")
+    << std::endl << _("     --page-id-template=TEMPLATE")
     << std::endl << _("     --page-title-template=TEMPLATE")
+    << std::endl <<   "     --no-page-titles"
     << std::endl << _(" -d, --dpi=RESOLUTION")
     << std::endl <<   "     --guess-dpi"
     << std::endl <<   "     --media-box"
