@@ -15,6 +15,7 @@
 
 #include <algorithm>
 #include <cassert>
+#include <cstddef>
 #include <cstdlib>
 #include <iomanip>
 #include <iostream>
@@ -42,10 +43,13 @@
 #include "image-filter.hh"
 #include "paths.hh"
 #include "pdf-backend.hh"
+#include "pdf-document-map.hh"
 #include "pdf-dpi.hh"
 #include "pdf-unicode.hh"
 #include "sexpr.hh"
 #include "string-format.hh"
+#include "string-printf.hh"
+#include "string-utils.hh"
 #include "system.hh"
 #include "version.hh"
 #include "xmp.hh"
@@ -133,79 +137,42 @@ public:
   }
 };
 
-class DocumentMap
-{
-protected:
-  intmax_t byte_size;
-  const std::vector<const char *> &filenames;
-  std::vector<int> map;
-public:
-  DocumentMap(const std::vector<const char *> &filenames_)
-  : byte_size(0),
-    filenames(filenames_)
-  {
-    int n_all_pages = 0;
-    for (std::vector<const char*>::iterator it = config.filenames.begin(); it != config.filenames.end(); it++)
-    {
-      {
-        std::ifstream ifs(*it, std::ifstream::in | std::ifstream::binary);
-        ifs.seekg(0, std::ios::end);
-        this->byte_size += ifs.tellg();
-        ifs.close();
-      }
-      {
-        std::auto_ptr<pdf::Document> doc(new pdf::Document(*it));
-        int n_pages = doc->getNumPages();
-        this->map.push_back(n_all_pages);
-        n_all_pages += n_pages;
-      }
-    }
-    this->map.push_back(n_all_pages);
-  }
-
-  intmax_t get_byte_size(void)
-  {
-    return this->byte_size;
-  }
-
-  int get_n_pages(void)
-  {
-    return this->map.back();
-  }
-
-  void get(int n, const char *&filename, int &m)
-  {
-    assert(n >= 1);
-    size_t i = std::upper_bound(this->map.begin(), this->map.end(), n - 1) - this->map.begin() - 1;
-    filename = this->filenames.at(i);
-    m = n - this->map[i];
-  }
-};
-
 class Component
 {
 protected:
-  std::auto_ptr<std::string> title;
+  std::string title;
+  bool title_set;
   File *file;
 public:
-  explicit Component(File &file, const std::string *title = NULL)
-  : file(&file)
+  explicit Component(File &file, const std::string title = "")
+  : title(title), title_set(false),
+    file(&file)
   {
-    if (title != NULL)
-      this->title.reset(new std::string(*title));
     file.close();
   }
 
   Component(const Component &component)
-  : file(component.file)
+  : title(component.title),
+    title_set(component.title_set),
+    file(component.file)
   {
-    if (component.title.get() != NULL)
-      this->title.reset(new std::string(*component.title));
   }
 
-  const std::string * get_title() const
+  const std::string & get_title() const
   {
-    return this->title.get();
+    assert(this->title_set);
+    return this->title;
+  }
+
+  void set_title(const std::string &title)
+  {
+    this->title = title;
+    // TODO: issue a warning if the title contains null bytes
+    string::replace_all(
+      this->title, '\0',
+      "\xEF\xBF\xBD"  // U+FFFD REPLACEMENT CHARACTER
+    );
+    this->title_set = true;
   }
 
   const std::string & get_basename() const
@@ -213,9 +180,9 @@ public:
     return this->file->get_basename();
   }
 
-  size_t size()
+  File::streamoff size()
   {
-    size_t result;
+    File::streamoff result;
     this->file->reopen();
     result = this->file->size();
     this->file->close();
@@ -275,25 +242,25 @@ protected:
     return bindings;
   }
 
-  std::string *get_title(int n) const
-  {
-    if (config.page_title_template.get() == NULL)
-      return NULL;
-    string_format::Bindings bindings = this->get_bindings(n);
-    return new std::string(config.page_title_template->format(bindings));
-  }
-
   virtual File *create_file(const std::string &id)
   = 0;
 
 public:
+
+  std::string get_title(int n, const std::string &label) const
+  {
+    string_format::Bindings bindings = this->get_bindings(n);
+    bindings["label"] = label;
+    return config.page_title_template->format(bindings);
+  }
+
   virtual Component &operator[](int n)
   {
     std::vector<Component*>::reference tmpfile_ptr = this->components.at(n - 1);
     if (tmpfile_ptr == NULL)
     {
       this->files[n - 1] = this->create_file(this->get_file_name(n));
-      tmpfile_ptr = new Component(*this->files[n - 1], this->get_title(n));
+      tmpfile_ptr = new Component(*this->files[n - 1]);
     }
     return *tmpfile_ptr;
   }
@@ -301,7 +268,7 @@ public:
   std::string get_file_name(int n) const
   {
     string_format::Bindings bindings = this->get_bindings(n);
-    return config.pageid_template->format(bindings);
+    return config.page_id_template->format(bindings);
   }
 
   virtual ~ComponentList() throw ()
@@ -775,9 +742,9 @@ protected:
   std::auto_ptr<const TemporaryDirectory> directory;
   std::auto_ptr<TemporaryFile> shared_ant_file;
 
-  virtual File *create_file(const std::string &pageid)
+  virtual File *create_file(const std::string &page_id)
   {
-    return new TemporaryFile(*this->directory, pageid);
+    return new TemporaryFile(*this->directory, page_id);
   }
 public:
   explicit TemporaryComponentList(int n, const PageMap &page_map)
@@ -802,9 +769,9 @@ private:
   IndirectComponentList& operator=(const IndirectComponentList&); // not defined
 protected:
   const Directory &directory;
-  virtual File *create_file(const std::string &pageid)
+  virtual File *create_file(const std::string &page_id)
   {
-    return new File(this->directory, pageid);
+    return new File(this->directory, page_id);
   }
 public:
   IndirectComponentList(int n, const PageMap &page_map, const Directory &directory)
@@ -880,12 +847,12 @@ void DjVm::remember(const Component &component)
   if (this->known_ids.count(id) > 0)
     throw DuplicateId(id);
   this->known_ids.insert(id);
-  const std::string *title = component.get_title();
-  if (title != NULL)
+  const std::string &title = component.get_title();
+  if (title.length() > 0)
   {
-    if (this->known_titles.count(*title) > 0)
-      throw DuplicateTitle(*title);
-    this->known_titles.insert(*title);
+    if (this->known_titles.count(title) > 0)
+      throw DuplicateTitle(title);
+    this->known_titles.insert(title);
   }
 }
 
@@ -1082,44 +1049,44 @@ void IndirectDjVm::create(const std::vector<Component> &components, bool bare)
     if (shared_ant)
       bzz_file << '\3';
     for (std::vector<Component>::const_iterator it = components.begin(); it != components.end(); it++)
-      bzz_file << (it->get_title() == NULL ? '\001' : '\101');
+      bzz_file << (it->get_title().length() == 0 ? '\001' : '\101');
     if (shared_ant)
       bzz_file << djvu::shared_ant_file_name << '\0';
     for (std::vector<Component>::const_iterator it = components.begin(); it != components.end(); it++)
     {
       bzz_file << it->get_basename() << '\0';
-      const std::string *title = it->get_title();
-      if (title == NULL)
+      const std::string &title = it->get_title();
+      if (title.length() == 0)
         continue;
-      bzz_file << *title << '\0';
+      bzz_file << title << '\0';
     }
     bzz_file.close();
     DjVuCommand bzz("bzz");
     bzz << "-e" << bzz_file << "-";
     bzz(index_file);
   }
-  size_t dirm_off = this->index_file.size();
-  this->index_file.seekg(20, std::ios::beg);
+  File::streamoff dirm_off = this->index_file.size();
+  this->index_file.seekp(20, std::ios::beg);
   for (int i = 3; i >= 0; i--)
     this->index_file << static_cast<char>(((dirm_off - 24) >> (8 * i)) & 0xff);
   dirm_off += dirm_off & 1;
   if (!bare && this->outline_stream.get())
   {
     TemporaryFile bzz_file;
-    this->index_file.seekg(dirm_off, std::ios::beg);
+    this->index_file.seekp(dirm_off, std::ios::beg);
     index_file.write("NAVM\0\0\0", 8);
     bzz_file << this->outline_stream->str();
     bzz_file.close();
     DjVuCommand bzz("bzz");
     bzz << "-e" << bzz_file << "-";
     bzz(index_file);
-    size_t outline_off = index_file.size();
-    this->index_file.seekg(dirm_off + 4, std::ios::beg);
+    File::streamoff outline_off = index_file.size();
+    this->index_file.seekp(dirm_off + 4, std::ios::beg);
     for (int i = 3; i >= 0; i--)
       this->index_file << static_cast<char>(((outline_off - dirm_off - 8) >> (8 * i)) & 0xff);
   }
-  size_t off = this->index_file.size();
-  this->index_file.seekg(8, std::ios::beg);
+  File::streamoff off = this->index_file.size();
+  this->index_file.seekp(8, std::ios::beg);
   for (int i = 3; i >= 0; i--)
     this->index_file << static_cast<char>(((off - 12) >> (8 * i)) & 0xff);
   this->index_file.close();
@@ -1250,7 +1217,7 @@ static int xmain(int argc, char * const argv[])
   pdf::Environment environment;
   environment.set_antialias(config.antialias);
 
-  DocumentMap document_map(config.filenames);
+  pdf::DocumentMap document_map(config.filenames);
   intmax_t pdf_byte_size = document_map.get_byte_size();
 
   pdf::splash::Color paper_color;
@@ -1356,9 +1323,16 @@ static int xmain(int argc, char * const argv[])
       throw DuplicatePage(n);
     page_map.set(n, i);
     page_numbers.push_back(n);
-    *djvm << (*page_files)[n];
     i++;
   }
+  for (std::vector<int>::const_iterator np = page_numbers.begin(); np != page_numbers.end(); np++)
+  {
+    Component &component = (*page_files)[*np];
+    component.set_title(
+       page_files->get_title(*np, document_map.get(*np).label)
+    );
+    *djvm << component;
+  }
 
   if (page_numbers.size() == 0)
     throw Config::NoPagesSelected();
@@ -1374,9 +1348,10 @@ static int xmain(int argc, char * const argv[])
   for (size_t i = 0; i < page_numbers.size(); i++)
   try
   {
-    int m, n = page_numbers[i];
-    const char *new_filename;
-    document_map.get(n, new_filename, m);
+    int n = page_numbers[i];
+    pdf::PageInfo pi = document_map.get(n);
+    const char * new_filename = pi.path;
+    int m = pi.local_pageno;
     if (new_filename != doc_filename)
     {
       doc_filename = new_filename;
@@ -1652,7 +1627,7 @@ static int xmain(int argc, char * const argv[])
         {
           xmp_bytes = xmp::transform(xmp_bytes, metadata);
         }
-        catch (xmp::XmlError &ex)
+        catch (xmp::Error &ex)
         {
           debug(1) << string_printf(_("Warning: %s"), ex.what()) << std::endl;
         }
