@@ -2,21 +2,28 @@
 
 # Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
 #
-# This package is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of pdfdjvu.
+#
+# pdf2djvu is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# pdf2djvu is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
 
 import unicodedata
 
-from common import (
+from tools import (
     case,
     re,
 )
 
 class test(case):
-    '''
-    fixed in [18b2ae04de2f], [fa7d4addf18e]
-    '''
+    # + fixed in 0.4.9 [18b2ae04de2f]
+    # + fixed in 0.4.11 [fa7d4addf18e]
+
     text = u'bar\N{LATIN SMALL LETTER DZ}o'
     text_nfkc = unicodedata.normalize('NFKC', text).encode('UTF-8')
     text_no_nfkc = text.encode('UTF-8')
